// common.c -- Defines some global functions.
//             From JamesM's kernel development tutorials.

#include "Common.h"

// Write a byte out to the specified port.
void outb(u16int port, u8int value)
{
    asm volatile ("outb %1, %0" : : "dN" (port), "a" (value));
}

// Read a byte from the specified port.
u8int inb(u16int port)
{
    u8int ret;
    asm volatile("inb %1, %0" : "=a" (ret) : "dN" (port));
    return ret;
}

// Read a word from the specified port.
u16int inw(u16int port)
{
    u16int ret;
    asm volatile ("inw %1, %0" : "=a" (ret) : "dN" (port));
    return ret;
}

// Copy len bytes from src to dest.
void memcpy(u8int *dest, const u8int *src, u32int len)
{
    const u8int *sp = (const u8int *)src;
    u8int *dp = (u8int *)dest;
    for(; len != 0; len--) *dp++ = *sp++;
}

// Write len copies of val into dest.
void memset(u8int *dest, u8int val, u32int len)
{
    u8int *temp = (u8int *)dest;
    for ( ; len != 0; len--) *temp++ = val;
}

// Compare two strings. Should return -1 if 
// str1 < str2, 0 if they are equal or 1 otherwise.
int strcmp(char *str1, char *str2)
{
    int i = 0;
    int failed = 0;    
    if(strlen(str1) < strlen(str2))
        failed = -1;
    else if(strlen(str1) < strlen(str2))
        failed = 1;
    else
    {
        while(str1[i] != '\0' && str2[i] != '\0')
        {
            if(str1[i] != str2[i])
            {
                failed = 1;
                break;
            }
            i++;
        }
    }
    return failed;
}

// Copy the NULL-terminated string src into dest, and
// return dest.
char *strcpy(char *dest, const char *src)
{
    char *dp = (char*)dest;
    char *sp = (char*)src;
    do
    {
      *dp++ = *sp++;
    }
    while (*sp != '\0');
}

// Concatenate the NULL-terminated string src onto
// the end of dest, and return dest.
char *strcat(char *dest, const char *src)
{
    char *dp = (char*)dest;
    char *sp = (char*)src;
    while (*dp != '\0')
    {
        *dp = *dp++;
    }

    do
    {
        *dp++ = *sp++;
    }
    while (*sp != '\0');
    return dest;
}

// Returns length of string.
int strlen(char *src)
{
   int i = 0;
   char *sp = (char*)src;
   while(sp != '\0')
   {
      i++; sp++;
   }
   return i;
}
